#include "openvpn_smartcard_key_wrapping.c"
#include <openssl/pem.h>

static void
create_import_apdu_with_binary_key(const unsigned char *key_data, BYTE *apdu, uint8_t key_type)
{
    // smartcard constants
    const BYTE INS = 0x01;
    const BYTE P1 = key_type;

    // defining offsets
    const uint DATA_OFF = 0x05;

    apdu[CLA_OFF] = CLA;
    apdu[INS_OFF] = INS;
    apdu[P1_OFF] = P1;
    apdu[P2_OFF] = 0x00;
    apdu[LC_OFF] = TLS_CRYPT_V2_KEY_LEN;
    memcpy(apdu + DATA_OFF, key_data, TLS_CRYPT_V2_KEY_LEN);
}

int import_key(unsigned char *key)
{
    LONG rv;
    struct CardConnectionContext card_ctx;
    BYTE import_apdu[APDU_LEN_WITHOUT_DATA + TLS_CRYPT_V2_KEY_LEN];
    uint16_t key_len;
    struct Request importKeyRequest = {import_apdu, sizeof(import_apdu)};
    BYTE responseBuffer[2];
    struct Request responseRequest = {responseBuffer, sizeof(responseBuffer)};

    rv = connectToCard(&card_ctx);
    if (rv != 0)
    {
        puts("Conencting to card failed");
        goto end;
    }

    rv = selectApp(&card_ctx);
    if (rv != 0)
    {
        puts("Selecting applet failed");
        goto end;
    }

    // AES Key
    create_import_apdu_with_binary_key(key, import_apdu, AES_KEY);
    rv = sendAPDU(&card_ctx, &importKeyRequest, &responseRequest);
    ERROR_CHECK(rv != EXIT_SUCCESS, "Transmit failed");
    RESPONSE_CHECK(responseRequest, "AES KEY import failed");

    // HMAC Key
    create_import_apdu_with_binary_key(key + 64, import_apdu, HMAC_KEY);
    rv = sendAPDU(&card_ctx, &importKeyRequest, &responseRequest);
    ERROR_CHECK(rv != EXIT_SUCCESS, "Transmit failed");
    RESPONSE_CHECK(responseRequest, "HMAC key import failed");

    unselectApp(&card_ctx);
    disconnectFromCard(&card_ctx);
    puts("Import key file succeeded!");
end:
    releaseContext(&card_ctx);
    return 0;
}

int main(int argc, char *argv[])
{
    if (argc != 2)
    {
        puts("USAGE: ./ImportKeyFile <server_key_file_path>");
    }

    char *key_file_path = argv[1];
    FILE *key_file = fopen(key_file_path, "r");
    char *name, *header;
    unsigned char *key_data;
    long len;
    int ret = PEM_read(key_file, &name, &header, &key_data, &len);
    if (! ret || len != 128)
    {
        puts("Error reading key file");
    }

    import_key(key_data);

    free(name);
    free(header);
    free(key_data);
}